@@ -119,6 +119,16 @@ typedef enum {
  */
 #define GST_STATE_PENDING(elem)		(GST_ELEMENT_CAST(elem)->pending_state)
 
+/**
+ * GST_STATE_TARGET:
+ * @elem: a #GstElement to return the target state for.
+ *
+ * This macro returns the target #GstState of the element.
+ *
+ * Since: 0.10.13
+ */
+#define GST_STATE_TARGET(elem)		(GST_ELEMENT_CAST(elem)->abidata.ABI.target_state)
+
 /**
  * GST_STATE_RETURN:
  * @elem: a #GstElement to return the last state result for.
@@ -254,7 +264,7 @@ typedef enum
 
 /**
  * GST_ELEMENT_ERROR:
- * @el:     the element that throws the error
+ * @el:     the element that generates the error
  * @domain: like CORE, LIBRARY, RESOURCE or STREAM (see #GstGError)
  * @code:   error code defined for that domain (see #GstGError)
  * @text:   the message to display (format string and args enclosed in
@@ -263,7 +273,7 @@ typedef enum
             enclosed in parentheses)
  *
  * Utility function that elements can use in case they encountered a fatal
- * data processing error. The pipeline will throw an error signal and the
+ * data processing error. The pipeline will post an error message and the
  * application will be requested to stop further media processing.
  */
 #define GST_ELEMENT_ERROR(el, domain, code, text, debug)		\
@@ -281,16 +291,16 @@ G_STMT_START {								\
 
 /**
  * GST_ELEMENT_WARNING:
- * @el:     the element that throws the error
- * @domain: like CORE, LIBRARY, RESOURCE or STREAM (see #GstGError)
- * @code:   error code defined for that domain (see #GstGError)
+ * @el:     the element that generates the warning
+ * @domain: like CORE, LIBRARY, RESOURCE or STREAM (see #gstreamer-GstGError)
+ * @code:   error code defined for that domain (see #gstreamer-GstGError)
  * @text:   the message to display (format string and args enclosed in
             parentheses)
  * @debug:  debugging information for the message (format string and args
             enclosed in parentheses)
  *
  * Utility function that elements can use in case they encountered a non-fatal
- * data processing problem. The pipeline will throw a warning signal and the
+ * data processing problem. The pipeline will post a warning message and the
  * application will be informed.
  */
 #define GST_ELEMENT_WARNING(el, domain, code, text, debug)		\
@@ -306,6 +316,36 @@ G_STMT_START {								\
   __txt, __dbg, __FILE__, GST_FUNCTION, __LINE__);			\
 } G_STMT_END
 
+/**
+ * GST_ELEMENT_INFO:
+ * @el:     the element that generates the information
+ * @domain: like CORE, LIBRARY, RESOURCE or STREAM (see #gstreamer-GstGError)
+ * @code:   error code defined for that domain (see #gstreamer-GstGError)
+ * @text:   the message to display (format string and args enclosed in
+            parentheses)
+ * @debug:  debugging information for the message (format string and args
+            enclosed in parentheses)
+ *
+ * Utility function that elements can use in case they want to inform
+ * the application of something noteworthy that is not an error.
+ * The pipeline will post a warning message and the
+ * application will be informed.
+ *
+ * Since: 0.10.12
+ */
+#define GST_ELEMENT_INFO(el, domain, code, text, debug)			\
+G_STMT_START {								\
+  gchar *__txt = _gst_element_error_printf text;			\
+  gchar *__dbg = _gst_element_error_printf debug;			\
+  if (__txt)								\
+    GST_INFO_OBJECT (el, "info: %s", __txt);				\
+  if (__dbg)								\
+    GST_INFO_OBJECT (el, "info: %s", __dbg);				\
+  gst_element_message_full (GST_ELEMENT(el), GST_MESSAGE_INFO,		\
+    GST_ ## domain ## _ERROR, GST_ ## domain ## _ERROR_ ## code,	\
+  __txt, __dbg, __FILE__, GST_FUNCTION, __LINE__);			\
+} G_STMT_END
+
 /* the state change mutexes and conds */
 /**
  * GST_STATE_GET_LOCK:
@@ -397,7 +437,14 @@ struct _GstElement
   guint32               pads_cookie;
 
   /*< private >*/
-  gpointer _gst_reserved[GST_PADDING];
+  union {
+    struct {
+      /* state set by application */
+      GstState              target_state;
+    } ABI;
+    /* adding + 0 to mark ABI change to be undone later */
+    gpointer _gst_reserved[GST_PADDING + 0];
+  } abidata;
 };
 
 /**
@@ -596,6 +643,8 @@ GstStateChangeReturn	gst_element_get_state		(GstElement * element,
 GstStateChangeReturn	gst_element_set_state		(GstElement *element, GstState state);
 
 void			gst_element_abort_state		(GstElement * element);
+GstStateChangeReturn    gst_element_change_state        (GstElement * element,
+		                                         GstStateChange transition);
 GstStateChangeReturn	gst_element_continue_state	(GstElement * element,
                                                          GstStateChangeReturn ret);
 void			gst_element_lost_state	        (GstElement * element);
