@@ -115,6 +115,7 @@ static GstEventQuarks event_quarks[] = {
   {GST_EVENT_QOS, "qos", 0},
   {GST_EVENT_SEEK, "seek", 0},
   {GST_EVENT_NAVIGATION, "navigation", 0},
+  {GST_EVENT_LATENCY, "latency", 0},
   {GST_EVENT_CUSTOM_UPSTREAM, "custom-upstream", 0},
   {GST_EVENT_CUSTOM_DOWNSTREAM, "custom-downstream", 0},
   {GST_EVENT_CUSTOM_DOWNSTREAM_OOB, "custom-downstream-oob", 0},
@@ -350,20 +351,20 @@ gst_event_get_structure (GstEvent * event)
 /**
  * gst_event_new_flush_start:
  *
- * Allocate a new flush start event. The flush start event can be send
+ * Allocate a new flush start event. The flush start event can be sent
  * upstream and downstream and travels out-of-bounds with the dataflow.
  *
  * It marks pads as being flushing and will make them return
  * #GST_FLOW_WRONG_STATE when used for data flow with gst_pad_push(),
  * gst_pad_chain(), gst_pad_alloc_buffer(), gst_pad_get_range() and
  * gst_pad_pull_range(). Any event (except a #GST_EVENT_FLUSH_STOP) received
- * on a flushing pad will return %FALSE immediatly.
+ * on a flushing pad will return %FALSE immediately.
  *
- * Elements unlock and blocking functions and exit their streaming functions
- * as fast as possible.
+ * Elements should unlock any blocking functions and exit their streaming
+ * functions as fast as possible when this event is received.
  *
- * This event is typically generated after a seek to minimize the latency
- * after the seek.
+ * This event is typically generated after a seek to flush out all queued data
+ * in the pipeline so that the new media is played as soon as possible.
  *
  * Returns: A new flush start event.
  */
@@ -376,9 +377,9 @@ gst_event_new_flush_start (void)
 /**
  * gst_event_new_flush_stop:
  *
- * Allocate a new flush stop event. The flush start event can be send
+ * Allocate a new flush stop event. The flush stop event can be sent
  * upstream and downstream and travels out-of-bounds with the dataflow.
- * It is typically send after sending a FLUSH_START event to make the
+ * It is typically sent after sending a FLUSH_START event to make the
  * pads accept data again.
  *
  * Elements can process this event synchronized with the dataflow since
@@ -407,7 +408,7 @@ gst_event_new_flush_stop (void)
  * which will then post the #GST_MESSAGE_EOS on the bus after they have
  * finished playing any buffered data.
  *
- * When all sinks have posted an EOS message, the EOS message is
+ * When all sinks have posted an EOS message, an EOS message is
  * forwarded to the application.
  *
  * Returns: The new EOS event.
@@ -912,3 +913,52 @@ gst_event_new_navigation (GstStructure * structure)
 
   return gst_event_new_custom (GST_EVENT_NAVIGATION, structure);
 }
+
+/**
+ * gst_event_new_latency:
+ * @latency: the new latency value
+ *
+ * Create a new latency event. The event is sent upstream from the sinks and
+ * notifies elements that they should add an additional @latency to the
+ * timestamps before synchronising against the clock.
+ *
+ * The latency is mostly used in live sinks and is always expressed in
+ * the time format.
+ *
+ * Returns: a new #GstEvent
+ *
+ * Since: 0.10.12
+ */
+GstEvent *
+gst_event_new_latency (GstClockTime latency)
+{
+  GST_CAT_INFO (GST_CAT_EVENT,
+      "creating latency event %" GST_TIME_FORMAT, GST_TIME_ARGS (latency));
+
+  return gst_event_new_custom (GST_EVENT_LATENCY,
+      gst_structure_new ("GstEventLatency",
+          "latency", G_TYPE_UINT64, latency, NULL));
+}
+
+/**
+ * gst_event_parse_latency:
+ * @event: The event to query
+ * @latency: A pointer to store the latency in.
+ *
+ * Get the latency in the latency event.
+ *
+ * Since: 0.10.12
+ */
+void
+gst_event_parse_latency (GstEvent * event, GstClockTime * latency)
+{
+  const GstStructure *structure;
+
+  g_return_if_fail (GST_IS_EVENT (event));
+  g_return_if_fail (GST_EVENT_TYPE (event) == GST_EVENT_LATENCY);
+
+  structure = gst_event_get_structure (event);
+  if (latency)
+    *latency =
+        g_value_get_uint64 (gst_structure_get_value (structure, "latency"));
+}
