@@ -34,6 +34,7 @@
 
 #include "gstghostpad.h"
 #include "gstutils.h"
+#include "gsterror.h"
 #include "gstinfo.h"
 #include "gstparse.h"
 #include "gst-i18n-lib.h"
@@ -99,7 +100,7 @@ gst_util_set_value_from_string (GValue * value, const gchar * value_str)
 
   switch (G_VALUE_TYPE (value)) {
     case G_TYPE_STRING:
-      g_value_set_string (value, g_strdup (value_str));
+      g_value_set_string (value, value_str);
       break;
     case G_TYPE_ENUM:
     case G_TYPE_INT:{
@@ -987,53 +988,83 @@ gst_element_get_compatible_pad (GstElement * element, GstPad * pad,
  *
  * Returns: a string with the name of the state.
  */
-const gchar *
+G_CONST_RETURN gchar *
 gst_element_state_get_name (GstState state)
 {
   switch (state) {
 #ifdef GST_DEBUG_COLOR
     case GST_STATE_VOID_PENDING:
       return "VOID_PENDING";
-      break;
     case GST_STATE_NULL:
       return "\033[01;34mNULL\033[00m";
-      break;
     case GST_STATE_READY:
       return "\033[01;31mREADY\033[00m";
-      break;
     case GST_STATE_PLAYING:
       return "\033[01;32mPLAYING\033[00m";
-      break;
     case GST_STATE_PAUSED:
       return "\033[01;33mPAUSED\033[00m";
-      break;
     default:
       /* This is a memory leak */
       return g_strdup_printf ("\033[01;35;41mUNKNOWN!\033[00m(%d)", state);
 #else
     case GST_STATE_VOID_PENDING:
       return "VOID_PENDING";
-      break;
     case GST_STATE_NULL:
       return "NULL";
-      break;
     case GST_STATE_READY:
       return "READY";
-      break;
     case GST_STATE_PLAYING:
       return "PLAYING";
-      break;
     case GST_STATE_PAUSED:
       return "PAUSED";
-      break;
     default:
       /* This is a memory leak */
       return g_strdup_printf ("UNKNOWN!(%d)", state);
 #endif
   }
-  return "";
 }
 
+/**
+ * gst_element_state_change_return_get_name:
+ * @state_ret: a #GstStateChangeReturn to get the name of.
+ *
+ * Gets a string representing the given state change result.
+ *
+ * Returns: a string with the name of the state change result.
+ */
+G_CONST_RETURN gchar *
+gst_element_state_change_return_get_name (GstStateChangeReturn state_ret)
+{
+  switch (state_ret) {
+#ifdef GST_DEBUG_COLOR
+    case GST_STATE_CHANGE_FAILURE:
+      return "\033[01;31mFAILURE\033[00m";
+    case GST_STATE_CHANGE_SUCCESS:
+      return "\033[01;32mSUCCESS\033[00m";
+    case GST_STATE_CHANGE_ASYNC:
+      return "\033[01;33mASYNC\033[00m";
+    case GST_STATE_CHANGE_NO_PREROLL:
+      return "\033[01;34mNO_PREROLL\033[00m";
+    default:
+      /* This is a memory leak */
+      return g_strdup_printf ("\033[01;35;41mUNKNOWN!\033[00m(%d)", state_ret);
+#else
+    case GST_STATE_CHANGE_FAILURE:
+      return "FAILURE";
+    case GST_STATE_CHANGE_SUCCESS:
+      return "SUCCESS";
+    case GST_STATE_CHANGE_ASYNC:
+      return "ASYNC";
+    case GST_STATE_CHANGE_NO_PREROLL:
+      return "NO PREROLL";
+    default:
+      /* This is a memory leak */
+      return g_strdup_printf ("UNKNOWN!(%d)", state_ret);
+#endif
+  }
+}
+
+
 /**
  * gst_element_factory_can_src_caps :
  * @factory: factory to query
@@ -1214,6 +1245,17 @@ prepare_link_maybe_ghosting (GstPad ** src, GstPad ** sink,
   e1 = GST_OBJECT_PARENT (*src);
   e2 = GST_OBJECT_PARENT (*sink);
 
+  if (G_UNLIKELY (e1 == NULL)) {
+    GST_WARNING ("Trying to ghost a pad that doesn't have a parent: %"
+        GST_PTR_FORMAT, *src);
+    return FALSE;
+  }
+  if (G_UNLIKELY (e2 == NULL)) {
+    GST_WARNING ("Trying to ghost a pad that doesn't have a parent: %"
+        GST_PTR_FORMAT, *sink);
+    return FALSE;
+  }
+
   if (GST_OBJECT_PARENT (e1) == GST_OBJECT_PARENT (e2)) {
     GST_CAT_INFO (GST_CAT_PADS, "%s and %s in same bin, no need for ghost pads",
         GST_OBJECT_NAME (e1), GST_OBJECT_NAME (e2));
@@ -1226,9 +1268,8 @@ prepare_link_maybe_ghosting (GstPad ** src, GstPad ** sink,
   /* we need to setup some ghost pads */
   root = find_common_root (e1, e2);
   if (!root) {
-    g_warning
-        ("Trying to connect elements that don't share a common ancestor: %s and %s\n",
-        GST_ELEMENT_NAME (e1), GST_ELEMENT_NAME (e2));
+    g_warning ("Trying to connect elements that don't share a common "
+        "ancestor: %s and %s", GST_ELEMENT_NAME (e1), GST_ELEMENT_NAME (e2));
     return FALSE;
   }
 
@@ -1482,13 +1523,8 @@ gst_element_link_pads (GstElement * src, const gchar * srcpadname,
     GST_CAT_DEBUG (GST_CAT_ELEMENT_PADS, "no link possible from %s to %s:%s",
         GST_ELEMENT_NAME (src), GST_DEBUG_PAD_NAME (destpad));
     gst_object_unref (destpad);
-    if (srcpad)
-      gst_object_unref (srcpad);
     return FALSE;
   } else {
-    if (srcpad)
-      gst_object_unref (srcpad);
-    srcpad = NULL;
     if (destpad)
       gst_object_unref (destpad);
     destpad = NULL;
@@ -1598,8 +1634,9 @@ gst_element_link_pads_filtered (GstElement * src, const gchar * srcpadname,
       return TRUE;
     } else {
       GST_INFO ("Could not link elements");
+      gst_element_set_state (capsfilter, GST_STATE_NULL);
+      /* this will unlink and unref as appropriate */
       gst_bin_remove (GST_BIN (GST_OBJECT_PARENT (capsfilter)), capsfilter);
-      /* will unref and unlink as appropriate */
       return FALSE;
     }
   } else {
@@ -1615,8 +1652,12 @@ gst_element_link_pads_filtered (GstElement * src, const gchar * srcpadname,
  * Links @src to @dest. The link must be from source to
  * destination; the other direction will not be tried. The function looks for
  * existing pads that aren't linked yet. It will request new pads if necessary.
+ * Such pads need to be released manualy when unlinking.
  * If multiple links are possible, only one is established.
  *
+ * Make sure you have added your elements to a bin or pipeline with
+ * gst_bin_add() before trying to link them.
+ *
  * Returns: TRUE if the elements could be linked, FALSE otherwise.
  */
 gboolean
@@ -1632,6 +1673,8 @@ gst_element_link (GstElement * src, GstElement * dest)
  * @...: the NULL-terminated list of elements to link in order.
  *
  * Chain together a series of elements. Uses gst_element_link().
+ * Make sure you have added your elements to a bin or pipeline with
+ * gst_bin_add() before trying to link them.
  *
  * Returns: TRUE on success, FALSE otherwise.
  */
@@ -1670,6 +1713,9 @@ gst_element_link_many (GstElement * element_1, GstElement * element_2, ...)
  * existing pads that aren't linked yet. It will request new pads if necessary.
  * If multiple links are possible, only one is established.
  *
+ * Make sure you have added your elements to a bin or pipeline with
+ * gst_bin_add() before trying to link them.
+ *
  * Returns: TRUE if the pads could be linked, FALSE otherwise.
  */
 gboolean
@@ -1756,6 +1802,9 @@ gst_element_unlink_many (GstElement * element_1, GstElement * element_2, ...)
  *
  * Unlinks all source pads of the source element with all sink pads
  * of the sink element to which they are linked.
+ *
+ * If the link has been made using gst_element_link(), it could have created an
+ * requestpad, which has to be released using gst_element_release_request_pad().
  */
 void
 gst_element_unlink (GstElement * src, GstElement * dest)
@@ -1811,6 +1860,7 @@ gst_element_unlink (GstElement * src, GstElement * dest)
         break;
     }
   }
+  gst_iterator_free (pads);
 }
 
 /**
@@ -1929,13 +1979,20 @@ gst_element_query_convert (GstElement * element, GstFormat src_format,
  * Simple API to perform a seek on the given element, meaning it just seeks
  * to the given position relative to the start of the stream. For more complex
  * operations like segment seeks (e.g. for looping) or changing the playback
- * rate or seeking relative to the current position or seeking relative to
- * the end of the stream you should use gst_element_seek ().
+ * rate or seeking relative to the last configured playback segment you should
+ * use gst_element_seek().
  *
- * Note that seeking is usually only possible in PAUSED or PLAYING state.
+ * In a completely prerolled PAUSED or PLAYING pipeline, seeking is always
+ * guaranteed to return %TRUE on a seekable media type or %FALSE when the media
+ * type is certainly not seekable (such as a live stream).
  *
- * Returns: TRUE if the seek operation succeeded (the seek
- *          might not always be executed instantly though)
+ * Some elements allow for seeking in the READY state, in this
+ * case they will store the seek event and execute it when they are put to
+ * PAUSED. If the element supports seek in READY, it will always return %TRUE when
+ * it receives the event in the READY state.
+ *
+ * Returns: %TRUE if the seek operation succeeded (the seek might not always be
+ * executed instantly though)
  *
  * Since: 0.10.7
  */
@@ -2045,6 +2102,7 @@ gst_pad_get_fixed_caps_func (GstPad * pad)
 
   g_return_val_if_fail (GST_IS_PAD (pad), NULL);
 
+  GST_OBJECT_LOCK (pad);
   if (GST_PAD_CAPS (pad)) {
     result = GST_PAD_CAPS (pad);
 
@@ -2069,6 +2127,8 @@ gst_pad_get_fixed_caps_func (GstPad * pad)
   result = gst_caps_new_empty ();
 
 done:
+  GST_OBJECT_UNLOCK (pad);
+
   return result;
 }
 
@@ -2358,18 +2418,16 @@ gst_buffer_join (GstBuffer * buf1, GstBuffer * buf2)
  * Copies additional information (the timestamp, duration, and offset start
  * and end) from one buffer to the other.
  *
- * This function does not copy any buffer flags or caps.
+ * This function does not copy any buffer flags or caps and is equivalent to
+ * gst_buffer_copy_metadata(@dest, @src, GST_BUFFER_COPY_TIMESTAMPS).
+ *
+ * Deprecated: use gst_buffer_copy_metadata() instead, it provides more
+ * control.
  */
 void
 gst_buffer_stamp (GstBuffer * dest, const GstBuffer * src)
 {
-  g_return_if_fail (dest != NULL);
-  g_return_if_fail (src != NULL);
-
-  GST_BUFFER_TIMESTAMP (dest) = GST_BUFFER_TIMESTAMP (src);
-  GST_BUFFER_DURATION (dest) = GST_BUFFER_DURATION (src);
-  GST_BUFFER_OFFSET (dest) = GST_BUFFER_OFFSET (src);
-  GST_BUFFER_OFFSET_END (dest) = GST_BUFFER_OFFSET_END (src);
+  gst_buffer_copy_metadata (dest, src, GST_BUFFER_COPY_TIMESTAMPS);
 }
 
 static gboolean
@@ -2420,18 +2478,40 @@ gst_pad_proxy_getcaps (GstPad * pad)
   if (element == NULL)
     return NULL;
 
-  iter = gst_element_iterate_pads (element);
-
+  /* value to hold the return, by default it holds ANY, the ref is taken by
+   * the GValue. */
   g_value_init (&ret, G_TYPE_POINTER);
   g_value_set_pointer (&ret, gst_caps_new_any ());
 
-  res = gst_iterator_fold (iter, (GstIteratorFoldFunction) intersect_caps_func,
-      &ret, pad);
+  iter = gst_element_iterate_pads (element);
+  while (1) {
+    res =
+        gst_iterator_fold (iter, (GstIteratorFoldFunction) intersect_caps_func,
+        &ret, pad);
+    switch (res) {
+      case GST_ITERATOR_RESYNC:
+        /* unref any value stored */
+        if ((caps = g_value_get_pointer (&ret)))
+          gst_caps_unref (caps);
+        /* need to reset the result again to ANY */
+        g_value_set_pointer (&ret, gst_caps_new_any ());
+        gst_iterator_resync (iter);
+        break;
+      case GST_ITERATOR_DONE:
+        /* all pads iterated, return collected value */
+        goto done;
+      default:
+        /* iterator returned _ERROR or premature end with _OK,
+         * mark an error and exit */
+        if ((caps = g_value_get_pointer (&ret)))
+          gst_caps_unref (caps);
+        g_value_set_pointer (&ret, NULL);
+        goto error;
+    }
+  }
+done:
   gst_iterator_free (iter);
 
-  if (res != GST_ITERATOR_DONE)
-    goto pads_changed;
-
   gst_object_unref (element);
 
   caps = g_value_get_pointer (&ret);
@@ -2443,10 +2523,11 @@ gst_pad_proxy_getcaps (GstPad * pad)
   return intersected;
 
   /* ERRORS */
-pads_changed:
+error:
   {
-    g_warning ("Pad list changed during capsnego for element %s",
+    g_warning ("Pad list returned error on element %s",
         GST_ELEMENT_NAME (element));
+    gst_iterator_free (iter);
     gst_object_unref (element);
     return NULL;
   }
@@ -2783,19 +2864,10 @@ gulong
 gst_pad_add_data_probe (GstPad * pad, GCallback handler, gpointer data)
 {
   gulong sigid;
-  gboolean was_ghost = FALSE;
 
   g_return_val_if_fail (GST_IS_PAD (pad), 0);
   g_return_val_if_fail (handler != NULL, 0);
 
-  if (GST_IS_GHOST_PAD (pad)) {
-    pad = gst_ghost_pad_get_target (GST_GHOST_PAD (pad));
-    if (!pad) {
-      return 0;
-    }
-    was_ghost = TRUE;
-  }
-
   GST_OBJECT_LOCK (pad);
   sigid = g_signal_connect (pad, "have-data", handler, data);
   GST_PAD_DO_EVENT_SIGNALS (pad)++;
@@ -2805,10 +2877,6 @@ gst_pad_add_data_probe (GstPad * pad, GCallback handler, gpointer data)
       GST_PAD_DO_BUFFER_SIGNALS (pad), GST_PAD_DO_EVENT_SIGNALS (pad));
   GST_OBJECT_UNLOCK (pad);
 
-  if (was_ghost) {
-    gst_object_unref (pad);
-  }
-
   return sigid;
 }
 
@@ -2827,19 +2895,10 @@ gulong
 gst_pad_add_event_probe (GstPad * pad, GCallback handler, gpointer data)
 {
   gulong sigid;
-  gboolean was_ghost = FALSE;
 
   g_return_val_if_fail (GST_IS_PAD (pad), 0);
   g_return_val_if_fail (handler != NULL, 0);
 
-  if (GST_IS_GHOST_PAD (pad)) {
-    pad = gst_ghost_pad_get_target (GST_GHOST_PAD (pad));
-    if (!pad) {
-      return 0;
-    }
-    was_ghost = TRUE;
-  }
-
   GST_OBJECT_LOCK (pad);
   sigid = g_signal_connect (pad, "have-data::event", handler, data);
   GST_PAD_DO_EVENT_SIGNALS (pad)++;
@@ -2847,10 +2906,6 @@ gst_pad_add_event_probe (GstPad * pad, GCallback handler, gpointer data)
       GST_DEBUG_PAD_NAME (pad), GST_PAD_DO_EVENT_SIGNALS (pad));
   GST_OBJECT_UNLOCK (pad);
 
-  if (was_ghost) {
-    gst_object_unref (pad);
-  }
-
   return sigid;
 }
 
@@ -2869,19 +2924,10 @@ gulong
 gst_pad_add_buffer_probe (GstPad * pad, GCallback handler, gpointer data)
 {
   gulong sigid;
-  gboolean was_ghost = FALSE;
 
   g_return_val_if_fail (GST_IS_PAD (pad), 0);
   g_return_val_if_fail (handler != NULL, 0);
 
-  if (GST_IS_GHOST_PAD (pad)) {
-    pad = gst_ghost_pad_get_target (GST_GHOST_PAD (pad));
-    if (!pad) {
-      return 0;
-    }
-    was_ghost = TRUE;
-  }
-
   GST_OBJECT_LOCK (pad);
   sigid = g_signal_connect (pad, "have-data::buffer", handler, data);
   GST_PAD_DO_BUFFER_SIGNALS (pad)++;
@@ -2889,10 +2935,6 @@ gst_pad_add_buffer_probe (GstPad * pad, GCallback handler, gpointer data)
       GST_DEBUG_PAD_NAME (pad), GST_PAD_DO_BUFFER_SIGNALS (pad));
   GST_OBJECT_UNLOCK (pad);
 
-  if (was_ghost) {
-    gst_object_unref (pad);
-  }
-
   return sigid;
 }
 
@@ -2906,19 +2948,9 @@ gst_pad_add_buffer_probe (GstPad * pad, GCallback handler, gpointer data)
 void
 gst_pad_remove_data_probe (GstPad * pad, guint handler_id)
 {
-  gboolean was_ghost = FALSE;
-
   g_return_if_fail (GST_IS_PAD (pad));
   g_return_if_fail (handler_id > 0);
 
-  if (GST_IS_GHOST_PAD (pad)) {
-    pad = gst_ghost_pad_get_target (GST_GHOST_PAD (pad));
-    if (!pad) {
-      return;
-    }
-    was_ghost = TRUE;
-  }
-
   GST_OBJECT_LOCK (pad);
   g_signal_handler_disconnect (pad, handler_id);
   GST_PAD_DO_BUFFER_SIGNALS (pad)--;
@@ -2929,9 +2961,6 @@ gst_pad_remove_data_probe (GstPad * pad, guint handler_id)
       GST_PAD_DO_BUFFER_SIGNALS (pad));
   GST_OBJECT_UNLOCK (pad);
 
-  if (was_ghost) {
-    gst_object_unref (pad);
-  }
 }
 
 /**
@@ -2944,29 +2973,15 @@ gst_pad_remove_data_probe (GstPad * pad, guint handler_id)
 void
 gst_pad_remove_event_probe (GstPad * pad, guint handler_id)
 {
-  gboolean was_ghost = FALSE;
-
   g_return_if_fail (GST_IS_PAD (pad));
   g_return_if_fail (handler_id > 0);
 
-  if (GST_IS_GHOST_PAD (pad)) {
-    pad = gst_ghost_pad_get_target (GST_GHOST_PAD (pad));
-    if (!pad) {
-      return;
-    }
-    was_ghost = TRUE;
-  }
-
   GST_OBJECT_LOCK (pad);
   g_signal_handler_disconnect (pad, handler_id);
   GST_PAD_DO_EVENT_SIGNALS (pad)--;
   GST_DEBUG ("removed event probe from pad %s:%s, now %d event probes",
       GST_DEBUG_PAD_NAME (pad), GST_PAD_DO_EVENT_SIGNALS (pad));
   GST_OBJECT_UNLOCK (pad);
-
-  if (was_ghost) {
-    gst_object_unref (pad);
-  }
 }
 
 /**
@@ -2979,19 +2994,9 @@ gst_pad_remove_event_probe (GstPad * pad, guint handler_id)
 void
 gst_pad_remove_buffer_probe (GstPad * pad, guint handler_id)
 {
-  gboolean was_ghost = FALSE;
-
   g_return_if_fail (GST_IS_PAD (pad));
   g_return_if_fail (handler_id > 0);
 
-  if (GST_IS_GHOST_PAD (pad)) {
-    pad = gst_ghost_pad_get_target (GST_GHOST_PAD (pad));
-    if (!pad) {
-      return;
-    }
-    was_ghost = TRUE;
-  }
-
   GST_OBJECT_LOCK (pad);
   g_signal_handler_disconnect (pad, handler_id);
   GST_PAD_DO_BUFFER_SIGNALS (pad)--;
@@ -2999,9 +3004,6 @@ gst_pad_remove_buffer_probe (GstPad * pad, guint handler_id)
       GST_DEBUG_PAD_NAME (pad), GST_PAD_DO_BUFFER_SIGNALS (pad));
   GST_OBJECT_UNLOCK (pad);
 
-  if (was_ghost) {
-    gst_object_unref (pad);
-  }
 }
 
 /**
@@ -3180,7 +3182,6 @@ gst_bin_find_unconnected_pad (GstBin * bin, GstPadDirection direction)
   return pad;
 }
 
-#ifndef GST_DISABLE_PARSE
 /**
  * gst_parse_bin_from_description:
  * @bin_description: command line describing the bin
@@ -3207,6 +3208,7 @@ GstElement *
 gst_parse_bin_from_description (const gchar * bin_description,
     gboolean ghost_unconnected_pads, GError ** err)
 {
+#ifndef GST_DISABLE_PARSE
   GstPad *pad = NULL;
   GstBin *bin;
   gchar *desc;
@@ -3240,5 +3242,15 @@ gst_parse_bin_from_description (const gchar * bin_description,
   }
 
   return GST_ELEMENT (bin);
-}
+#else
+  gchar *msg;
+
+  GST_WARNING ("Disabled API called: gst_parse_bin_from_description()");
+
+  msg = gst_error_get_message (GST_CORE_ERROR, GST_CORE_ERROR_DISABLED);
+  g_set_error (err, GST_CORE_ERROR, GST_CORE_ERROR_DISABLED, "%s", msg);
+  g_free (msg);
+
+  return NULL;
 #endif
+}
