@@ -132,9 +132,17 @@ struct _GstBaseSrc {
  * @is_seekable: Check if the source can seek
  * @unlock: Unlock any pending access to the resource. Subclasses should
  *    unblock any blocked function ASAP
+ * @unlock_stop: Clear the previous unlock request. Subclasses should clear
+ *    any state they set during unlock(), such as clearing command queues. 
  * @event: Override this to implement custom event handling.
  * @create: Ask the subclass to create a buffer with offset and size.
  * @do_seek: Perform seeking on the resource to the indicated segment.
+ * @prepare_seek_segment: Prepare the GstSegment that will be passed to the 
+ *   do_seek vmethod for executing a seek request. Sub-classes should override 
+ *   this if they support seeking in formats other than the configured native 
+ *   format. By default, it tries to convert the seek arguments to the 
+ *   configured native format and prepare a segment in that format.
+ *   Since: 0.10.13
  * @query: Handle a requested query. 
  * @check_get_range: Check whether the source would support pull-based 
  *   operation if it were to be opened now. This vfunc is optional, but 
@@ -208,8 +216,16 @@ struct _GstBaseSrcClass {
   /* called if, in negotation, caps need fixating */
   void		(*fixate)	(GstBaseSrc *src, GstCaps *caps);
 
+  /* Clear any pending unlock request, as we succeeded in unlocking */
+  gboolean      (*unlock_stop)  (GstBaseSrc *src);
+
+  /* Prepare the segment on which to perform do_seek(), converting to the
+   * current basesrc format. */
+  gboolean      (*prepare_seek_segment) (GstBaseSrc *src, GstEvent *seek, 
+                                         GstSegment *segment); 
+
   /*< private >*/
-  gpointer       _gst_reserved[GST_PADDING_LARGE - 4];
+  gpointer       _gst_reserved[GST_PADDING_LARGE - 6];
 };
 
 GType gst_base_src_get_type (void);
@@ -221,6 +237,10 @@ gboolean	gst_base_src_is_live	   (GstBaseSrc *src);
 
 void		gst_base_src_set_format	   (GstBaseSrc *src, GstFormat format);
 
+gboolean        gst_base_src_query_latency (GstBaseSrc *src, gboolean * live,
+                                            GstClockTime * min_latency, 
+					    GstClockTime * max_latency);
+
 G_END_DECLS
 
 #endif /* __GST_BASE_SRC_H__ */
