@@ -230,6 +230,8 @@ struct _GstBaseTransformPrivate
   gboolean qos_enabled;
   gdouble proportion;
   GstClockTime earliest_time;
+  /* previous buffer had a discont */
+  gboolean discont;
 
   GstActivateMode pad_mode;
 };
@@ -970,10 +972,8 @@ gst_base_transform_prepare_output_buffer (GstBaseTransform * trans,
     if (copy_inbuf && gst_buffer_is_writable (*out_buf))
       memcpy (GST_BUFFER_DATA (*out_buf), GST_BUFFER_DATA (in_buf), out_size);
 
-    gst_buffer_stamp (*out_buf, in_buf);
-    GST_BUFFER_FLAGS (*out_buf) |= GST_BUFFER_FLAGS (in_buf) &
-        (GST_BUFFER_FLAG_PREROLL | GST_BUFFER_FLAG_DISCONT |
-        GST_BUFFER_FLAG_IN_CAPS | GST_BUFFER_FLAG_DELTA_UNIT);
+    gst_buffer_copy_metadata (*out_buf, in_buf,
+        GST_BUFFER_COPY_FLAGS | GST_BUFFER_COPY_TIMESTAMPS);
   }
 
 done:
@@ -1189,7 +1189,7 @@ not_configured:
     /* let the default allocator handle it... */
     GST_DEBUG_OBJECT (trans, "not configured");
     gst_buffer_replace (buf, NULL);
-    if (trans->passthrough || trans->always_in_place) {
+    if (trans->passthrough) {
       /* ...by calling alloc_buffer without setting caps on the src pad, which
        * will force negotiation in the chain function. */
       res = gst_pad_alloc_buffer (trans->srcpad, offset, size, caps, buf);
@@ -1204,7 +1204,7 @@ unknown_size:
     /* let the default allocator handle it... */
     GST_DEBUG_OBJECT (trans, "unknown size");
     gst_buffer_replace (buf, NULL);
-    if (trans->passthrough || trans->always_in_place) {
+    if (trans->passthrough) {
       /* ...by calling alloc_buffer without setting caps on the src pad, which
        * will force negotiation in the chain function. */
       res = gst_pad_alloc_buffer (trans->srcpad, offset, size, caps, buf);
@@ -1250,6 +1250,7 @@ gst_base_transform_sink_eventfunc (GstBaseTransform * trans, GstEvent * event)
       /* reset QoS parameters */
       trans->priv->proportion = 1.0;
       trans->priv->earliest_time = -1;
+      trans->priv->discont = FALSE;
       GST_OBJECT_UNLOCK (trans);
       /* we need new segment info after the flush. */
       trans->have_newsegment = FALSE;
@@ -1372,6 +1373,12 @@ gst_base_transform_handle_buffer (GstBaseTransform * trans, GstBuffer * inbuf,
   if (!trans->negotiated && !trans->passthrough && (bclass->set_caps != NULL))
     goto not_negotiated;
 
+  /* Set discont flag so we can mark the outgoing buffer */
+  if (GST_BUFFER_IS_DISCONT (inbuf)) {
+    GST_LOG_OBJECT (trans, "got DISCONT buffer %p", inbuf);
+    trans->priv->discont = TRUE;
+  }
+
   /* can only do QoS if the segment is in TIME */
   if (trans->segment.format != GST_FORMAT_TIME)
     goto no_qos;
@@ -1395,6 +1402,8 @@ gst_base_transform_handle_buffer (GstBaseTransform * trans, GstBuffer * inbuf,
       GST_CAT_DEBUG_OBJECT (GST_CAT_QOS, trans, "skipping transform: qostime %"
           GST_TIME_FORMAT " <= %" GST_TIME_FORMAT,
           GST_TIME_ARGS (qostime), GST_TIME_ARGS (earliest_time));
+      /* mark discont for next buffer */
+      trans->priv->discont = TRUE;
       goto skip;
     }
   }
@@ -1411,7 +1420,7 @@ no_qos:
 
     *outbuf = inbuf;
 
-    return ret;
+    goto done;
   }
 
   want_in_place = (bclass->transform_ip != NULL) && trans->always_in_place;
@@ -1477,6 +1486,7 @@ skip:
   if (*outbuf != inbuf)
     gst_buffer_unref (inbuf);
 
+done:
   return ret;
 
   /* ERRORS */
@@ -1556,23 +1566,52 @@ gst_base_transform_chain (GstPad * pad, GstBuffer * buffer)
 {
   GstBaseTransform *trans;
   GstFlowReturn ret;
+  GstClockTime last_stop = GST_CLOCK_TIME_NONE;
   GstBuffer *outbuf = NULL;
 
   trans = GST_BASE_TRANSFORM (gst_pad_get_parent (pad));
 
+  /* calculate end position of the incoming buffer */
+  if (GST_BUFFER_TIMESTAMP (buffer) != GST_CLOCK_TIME_NONE) {
+    if (GST_BUFFER_DURATION (buffer) != GST_CLOCK_TIME_NONE)
+      last_stop = GST_BUFFER_TIMESTAMP (buffer) + GST_BUFFER_DURATION (buffer);
+    else
+      last_stop = GST_BUFFER_TIMESTAMP (buffer);
+  }
+
   /* protect transform method and concurrent buffer alloc */
   g_mutex_lock (trans->transform_lock);
   ret = gst_base_transform_handle_buffer (trans, buffer, &outbuf);
   g_mutex_unlock (trans->transform_lock);
 
-  /* outbuf can be NULL, this means a dropped buffer */
+  /* outbuf can be NULL, this means a dropped buffer, if we have a buffer but
+   * GST_BASE_TRANSFORM_FLOW_DROPPED we will not push either. */
   if (outbuf != NULL) {
-    if ((ret == GST_FLOW_OK))
+    if ((ret == GST_FLOW_OK)) {
+      /* Remember last stop position */
+      if ((last_stop != GST_CLOCK_TIME_NONE) &&
+          (trans->segment.format == GST_FORMAT_TIME))
+        gst_segment_set_last_stop (&trans->segment, GST_FORMAT_TIME, last_stop);
+
+      /* apply DISCONT flag if the buffer is not yet marked as such */
+      if (trans->priv->discont) {
+        if (!GST_BUFFER_IS_DISCONT (outbuf)) {
+          outbuf = gst_buffer_make_metadata_writable (outbuf);
+          GST_BUFFER_FLAG_SET (outbuf, GST_BUFFER_FLAG_DISCONT);
+        }
+        trans->priv->discont = FALSE;
+      }
       ret = gst_pad_push (trans->srcpad, outbuf);
-    else
+    } else
       gst_buffer_unref (outbuf);
   }
 
+  /* convert internal flow to OK and mark discont for the next buffer. */
+  if (ret == GST_BASE_TRANSFORM_FLOW_DROPPED) {
+    trans->priv->discont = TRUE;
+    ret = GST_FLOW_OK;
+  }
+
   gst_object_unref (trans);
 
   return ret;
@@ -1641,9 +1680,15 @@ gst_base_transform_activate (GstBaseTransform * trans, gboolean active)
     gst_segment_init (&trans->segment, GST_FORMAT_UNDEFINED);
     trans->priv->proportion = 1.0;
     trans->priv->earliest_time = -1;
+    trans->priv->discont = FALSE;
 
     GST_OBJECT_UNLOCK (trans);
   } else {
+    /* We must make sure streaming has finished before resetting things
+     * and calling the ::stop vfunc */
+    GST_PAD_STREAM_LOCK (trans->sinkpad);
+    GST_PAD_STREAM_UNLOCK (trans->sinkpad);
+
     trans->have_same_caps = FALSE;
     /* We can only reset the passthrough mode if the instance told us to 
        handle it in configure_caps */
