@@ -83,6 +83,8 @@ static GstFlowReturn gst_file_sink_render (GstBaseSink * sink,
 
 static gboolean gst_file_sink_do_seek (GstFileSink * filesink,
     guint64 new_offset);
+static gboolean gst_file_sink_get_current_offset (GstFileSink * filesink,
+    guint64 * p_pos);
 
 static gboolean gst_file_sink_query (GstPad * pad, GstQuery * query);
 
@@ -242,7 +244,7 @@ gst_file_sink_open_file (GstFileSink * sink)
   if (sink->file == NULL)
     goto open_failed;
 
-  sink->data_written = 0;
+  sink->current_pos = 0;
   /* try to seek in the file to figure out if it is seekable */
   sink->seekable = gst_file_sink_do_seek (sink, 0);
 
@@ -302,7 +304,7 @@ gst_file_sink_query (GstPad * pad, GstQuery * query)
       switch (format) {
         case GST_FORMAT_DEFAULT:
         case GST_FORMAT_BYTES:
-          gst_query_set_position (query, GST_FORMAT_BYTES, self->data_written);
+          gst_query_set_position (query, GST_FORMAT_BYTES, self->current_pos);
           return TRUE;
         default:
           return FALSE;
@@ -317,9 +319,9 @@ gst_file_sink_query (GstPad * pad, GstQuery * query)
   }
 }
 
-#if HAVE_FSEEKO
+#ifdef HAVE_FSEEKO
 # define __GST_STDIO_SEEK_FUNCTION "fseeko"
-#elif G_OS_UNIX
+#elif defined (G_OS_UNIX)
 # define __GST_STDIO_SEEK_FUNCTION "lseek"
 #else
 # define __GST_STDIO_SEEK_FUNCTION "fseek"
@@ -334,10 +336,10 @@ gst_file_sink_do_seek (GstFileSink * filesink, guint64 new_offset)
   if (fflush (filesink->file))
     goto flush_failed;
 
-#if HAVE_FSEEKO
+#ifdef HAVE_FSEEKO
   if (fseeko (filesink->file, (off_t) new_offset, SEEK_SET) != 0)
     goto seek_failed;
-#elif G_OS_UNIX
+#elif defined (G_OS_UNIX)
   if (lseek (fileno (filesink->file), (off_t) new_offset,
           SEEK_SET) == (off_t) - 1)
     goto seek_failed;
@@ -346,6 +348,10 @@ gst_file_sink_do_seek (GstFileSink * filesink, guint64 new_offset)
     goto seek_failed;
 #endif
 
+  /* adjust position reporting after seek;
+   * presumably this should basically yield new_offset */
+  gst_file_sink_get_current_offset (filesink, &filesink->current_pos);
+
   return TRUE;
 
   /* ERRORS */
@@ -426,9 +432,9 @@ gst_file_sink_get_current_offset (GstFileSink * filesink, guint64 * p_pos)
 {
   off_t ret;
 
-#if HAVE_FTELLO
+#ifdef HAVE_FTELLO
   ret = ftello (filesink->file);
-#elif G_OS_UNIX
+#elif defined (G_OS_UNIX)
   if (fflush (filesink->file)) {
     GST_DEBUG_OBJECT (filesink, "Flush failed: %s", g_strerror (errno));
     /* ignore and continue */
@@ -438,7 +444,8 @@ gst_file_sink_get_current_offset (GstFileSink * filesink, guint64 * p_pos)
   ret = (off_t) ftell (filesink->file);
 #endif
 
-  *p_pos = (guint64) ret;
+  if (ret != (off_t) - 1)
+    *p_pos = (guint64) ret;
 
   return (ret != (off_t) - 1);
 }
@@ -447,32 +454,20 @@ static GstFlowReturn
 gst_file_sink_render (GstBaseSink * sink, GstBuffer * buffer)
 {
   GstFileSink *filesink;
-  guint64 cur_pos;
   guint size;
-  guint64 back_pending = 0;
 
   size = GST_BUFFER_SIZE (buffer);
 
   filesink = GST_FILE_SINK (sink);
 
-  if (filesink->seekable) {
-    if (!gst_file_sink_get_current_offset (filesink, &cur_pos))
-      goto handle_error;
-  } else {
-    cur_pos = filesink->data_written;
-  }
-
-  if (cur_pos < filesink->data_written)
-    back_pending = filesink->data_written - cur_pos;
-
   GST_DEBUG_OBJECT (filesink, "writing %u bytes at %" G_GUINT64_FORMAT,
-      size, cur_pos);
+      size, filesink->current_pos);
 
   if (size > 0 && GST_BUFFER_DATA (buffer) != NULL) {
     if (fwrite (GST_BUFFER_DATA (buffer), size, 1, filesink->file) != 1)
       goto handle_error;
 
-    filesink->data_written += size - back_pending;
+    filesink->current_pos += size;
   }
 
   return GST_FLOW_OK;
@@ -554,14 +549,21 @@ gst_file_sink_uri_set_uri (GstURIHandler * handler, const gchar * uri)
      * "location" with uri + 16 because it provides unescaping */
     location = gst_uri_get_location (tmp);
     g_free (tmp);
+  } else if (strcmp (uri, "file://") == 0) {
+    /* Special case for "file://" as this is used by some applications
+     *  to test with gst_element_make_from_uri if there's an element
+     *  that supports the URI protocol. */
+    gst_file_sink_set_location (sink, NULL);
+    return TRUE;
   } else {
     location = gst_uri_get_location (uri);
-    if (!location)
-      return FALSE;
-    if (!g_path_is_absolute (location)) {
-      g_free (location);
-      return FALSE;
-    }
+  }
+
+  if (!location)
+    return FALSE;
+  if (!g_path_is_absolute (location)) {
+    g_free (location);
+    return FALSE;
   }
 
   ret = gst_file_sink_set_location (sink, location);
