@@ -63,9 +63,12 @@ struct _GstIdentity {
   gboolean 	 dump;
   gboolean 	 sync;
   gboolean 	 check_perfect;
+  gboolean 	 check_imperfect_timestamp;
+  gboolean 	 check_imperfect_offset;
   gboolean	 single_segment;
   GstClockTime   prev_timestamp;
   GstClockTime   prev_duration;
+  guint64        prev_offset;
   guint64        prev_offset_end;
   gchar 	*last_message;
   guint64        offset;
