@@ -26,16 +26,34 @@
  * SECTION:element-queue
  * @short_description: Simple asynchronous data queue.
  *
- * Data is queued till max_level buffers have been stored. Any subsequent
- * buffers sent to this filter will block until free space becomes available in
- * the buffer. The queue will create a new thread on the source pad to
- * decouple the processing on sink and source pad.
+ * Data is queued until one of the limits specified by the
+ * #GstQueue:max-size-buffers, #GstQueue:max-size-bytes and/or
+ * #GstQueue:max-size-time properties has been reached. Any attempt to push
+ * more buffers into the queue will block the pushing thread until more space
+ * becomes available.
  *
- * You can query how many buffers are queued with the level argument.
+ * The queue will create a new thread on the source pad to decouple the
+ * processing on sink and source pad.
  *
- * The default queue length is set to 100.
+ * You can query how many buffers are queued by reading the
+ * #GstQueue:current-level-buffers property. You can track changes
+ * by connecting to the notify::current-level-buffers signal (which
+ * like all signals will be emitted from the streaming thread). The same
+ * applies to the #GstQueue:current-level-time and
+ * #GstQueue:current-level-bytes properties.
  *
- * The queue blocks by default.
+ * The default queue size limits are 200 buffers, 10MB of data, or
+ * one second worth of data, whichever is reached first.
+ *
+ * As said earlier, the queue blocks by default when one of the specified
+ * maximums (bytes, time, buffers) has been reached. You can set the
+ * #GstQueue:leaky property to specify that instead of blocking it should
+ * leak (drop) new or old buffers.
+ *
+ * The #GstQueue::underrun signal is emitted when the queue has less data than
+ * the specified minimum thresholds require (by default: when the queue is
+ * empty). The #GstQueue::overrun signal is emitted when the queue is filled
+ * up. Both signals are emitted from the context of the streaming thread.
  */
 
 #include "gst/gst_private.h"
@@ -63,7 +81,7 @@ GST_DEBUG_CATEGORY_STATIC (queue_dataflow);
   GST_CAT_LOG_OBJECT (queue_dataflow, queue, \
                       "(%s:%s) " msg ": %u of %u-%u buffers, %u of %u-%u " \
                       "bytes, %" G_GUINT64_FORMAT " of %" G_GUINT64_FORMAT \
-                      "-%" G_GUINT64_FORMAT " ns, %u elements", \
+                      "-%" G_GUINT64_FORMAT " ns, %u items", \
                       GST_DEBUG_PAD_NAME (pad), \
                       queue->cur_level.buffers, \
                       queue->min_threshold.buffers, \
@@ -81,13 +99,13 @@ static const GstElementDetails gst_queue_details = GST_ELEMENT_DETAILS ("Queue",
     "Simple data queue",
     "Erik Walthinsen <omega@cse.ogi.edu>");
 
-
 /* Queue signals and args */
 enum
 {
   SIGNAL_UNDERRUN,
   SIGNAL_RUNNING,
   SIGNAL_OVERRUN,
+  SIGNAL_PUSHING,
   LAST_SIGNAL
 };
 
@@ -109,14 +127,13 @@ enum
   /* FILL ME */
 };
 
+/* default property values */
+#define DEFAULT_MAX_SIZE_BUFFERS  200   /* 200 buffers */
+#define DEFAULT_MAX_SIZE_BYTES    (10 * 1024 * 1024)    /* 10 MB       */
+#define DEFAULT_MAX_SIZE_TIME     GST_SECOND    /* 1 second    */
+
 #define GST_QUEUE_MUTEX_LOCK(q) G_STMT_START {                          \
-  GST_CAT_LOG_OBJECT (queue_dataflow, q,                                \
-      "locking qlock from thread %p",                                   \
-      g_thread_self ());                                                \
   g_mutex_lock (q->qlock);                                              \
-  GST_CAT_LOG_OBJECT (queue_dataflow, q,                                \
-      "locked qlock from thread %p",                                    \
-      g_thread_self ());                                                \
 } G_STMT_END
 
 #define GST_QUEUE_MUTEX_LOCK_CHECK(q,label) G_STMT_START {              \
@@ -126,16 +143,47 @@ enum
 } G_STMT_END
 
 #define GST_QUEUE_MUTEX_UNLOCK(q) G_STMT_START {                        \
-  GST_CAT_LOG_OBJECT (queue_dataflow, q,                                \
-      "unlocking qlock from thread %p",                                 \
-      g_thread_self ());                                                \
   g_mutex_unlock (q->qlock);                                            \
 } G_STMT_END
 
+#define GST_QUEUE_WAIT_DEL_CHECK(q, label) G_STMT_START {               \
+  STATUS (queue, q->sinkpad, "wait for DEL");                           \
+  g_cond_wait (q->item_del, queue->qlock);                              \
+  if (q->srcresult != GST_FLOW_OK) {                                    \
+    STATUS (queue, q->srcpad, "received DEL wakeup");                   \
+    goto label;                                                         \
+  }                                                                     \
+  STATUS (queue, q->sinkpad, "received DEL");                           \
+} G_STMT_END
+
+#define GST_QUEUE_WAIT_ADD_CHECK(q, label) G_STMT_START {               \
+  STATUS (queue, q->srcpad, "wait for ADD");                            \
+  g_cond_wait (q->item_add, q->qlock);                                  \
+  if (q->srcresult != GST_FLOW_OK) {                                    \
+    STATUS (queue, q->srcpad, "received ADD wakeup");                   \
+    goto label;                                                         \
+  }                                                                     \
+  STATUS (queue, q->srcpad, "received ADD");                            \
+} G_STMT_END
+
+#define GST_QUEUE_SIGNAL_DEL(q) G_STMT_START {                          \
+  STATUS (q, q->srcpad, "signal DEL");                                  \
+  g_cond_signal (q->item_del);                                          \
+} G_STMT_END
+
+#define GST_QUEUE_SIGNAL_ADD(q) G_STMT_START {                          \
+  STATUS (q, q->sinkpad, "signal ADD");                                 \
+  g_cond_signal (q->item_add);                                          \
+} G_STMT_END
+
+#define _do_init(bla) \
+    GST_DEBUG_CATEGORY_INIT (queue_debug, "queue", 0, "queue element"); \
+    GST_DEBUG_CATEGORY_INIT (queue_dataflow, "queue_dataflow", 0, \
+        "dataflow inside the queue element");
+
+GST_BOILERPLATE_FULL (GstQueue, gst_queue, GstElement,
+    GST_TYPE_ELEMENT, _do_init);
 
-static void gst_queue_base_init (GstQueueClass * klass);
-static void gst_queue_class_init (GstQueueClass * klass);
-static void gst_queue_init (GstQueue * queue);
 static void gst_queue_finalize (GObject * object);
 
 static void gst_queue_set_property (GObject * object,
@@ -176,8 +224,9 @@ queue_leaky_get_type (void)
   static GType queue_leaky_type = 0;
   static const GEnumValue queue_leaky[] = {
     {GST_QUEUE_NO_LEAK, "Not Leaky", "no"},
-    {GST_QUEUE_LEAK_UPSTREAM, "Leaky on Upstream", "upstream"},
-    {GST_QUEUE_LEAK_DOWNSTREAM, "Leaky on Downstream", "downstream"},
+    {GST_QUEUE_LEAK_UPSTREAM, "Leaky on upstream (new buffers)", "upstream"},
+    {GST_QUEUE_LEAK_DOWNSTREAM, "Leaky on downstream (old buffers)",
+        "downstream"},
     {0, NULL, NULL},
   };
 
@@ -187,42 +236,12 @@ queue_leaky_get_type (void)
   return queue_leaky_type;
 }
 
-static GstElementClass *parent_class = NULL;
 static guint gst_queue_signals[LAST_SIGNAL] = { 0 };
 
-GType
-gst_queue_get_type (void)
-{
-  static GType queue_type = 0;
-
-  if (!queue_type) {
-    static const GTypeInfo queue_info = {
-      sizeof (GstQueueClass),
-      (GBaseInitFunc) gst_queue_base_init,
-      NULL,
-      (GClassInitFunc) gst_queue_class_init,
-      NULL,
-      NULL,
-      sizeof (GstQueue),
-      0,
-      (GInstanceInitFunc) gst_queue_init,
-      NULL
-    };
-
-    queue_type = g_type_register_static (GST_TYPE_ELEMENT,
-        "GstQueue", &queue_info, 0);
-    GST_DEBUG_CATEGORY_INIT (queue_debug, "queue", 0, "queue element");
-    GST_DEBUG_CATEGORY_INIT (queue_dataflow, "queue_dataflow", 0,
-        "dataflow inside the queue element");
-  }
-
-  return queue_type;
-}
-
 static void
-gst_queue_base_init (GstQueueClass * klass)
+gst_queue_base_init (gpointer g_class)
 {
-  GstElementClass *gstelement_class = GST_ELEMENT_CLASS (klass);
+  GstElementClass *gstelement_class = GST_ELEMENT_CLASS (g_class);
 
   gst_element_class_add_pad_template (gstelement_class,
       gst_static_pad_template_get (&srctemplate));
@@ -237,8 +256,6 @@ gst_queue_class_init (GstQueueClass * klass)
   GObjectClass *gobject_class = G_OBJECT_CLASS (klass);
   GstElementClass *gstelement_class = GST_ELEMENT_CLASS (klass);
 
-  parent_class = g_type_class_peek_parent (klass);
-
   gobject_class->set_property = GST_DEBUG_FUNCPTR (gst_queue_set_property);
   gobject_class->get_property = GST_DEBUG_FUNCPTR (gst_queue_get_property);
 
@@ -249,8 +266,8 @@ gst_queue_class_init (GstQueueClass * klass)
    *
    * Reports that the buffer became empty (underrun).
    * A buffer is empty if the total amount of data inside it (num-buffers, time,
-   * size) is lower than the boundary values which can be set through the GObject
-   * properties.
+   * size) is lower than the boundary values which can be set through the
+   * GObject properties.
    */
   gst_queue_signals[SIGNAL_UNDERRUN] =
       g_signal_new ("underrun", G_TYPE_FROM_CLASS (klass), G_SIGNAL_RUN_FIRST,
@@ -261,8 +278,8 @@ gst_queue_class_init (GstQueueClass * klass)
    * @queue: the queue instance
    *
    * Reports that enough (min-threshold) data is in the queue. Use this signal
-   * together with the underrun signal to pause the pipeline on underrun and wait
-   * for the queue to fill-up before resume playback.
+   * together with the underrun signal to pause the pipeline on underrun and
+   * wait for the queue to fill-up before resume playback.
    */
   gst_queue_signals[SIGNAL_RUNNING] =
       g_signal_new ("running", G_TYPE_FROM_CLASS (klass), G_SIGNAL_RUN_FIRST,
@@ -274,13 +291,24 @@ gst_queue_class_init (GstQueueClass * klass)
    *
    * Reports that the buffer became full (overrun).
    * A buffer is full if the total amount of data inside it (num-buffers, time,
-   * size) is higher than the boundary values which can be set through the GObject
-   * properties.
+   * size) is higher than the boundary values which can be set through the
+   * GObject properties.
    */
   gst_queue_signals[SIGNAL_OVERRUN] =
       g_signal_new ("overrun", G_TYPE_FROM_CLASS (klass), G_SIGNAL_RUN_FIRST,
       G_STRUCT_OFFSET (GstQueueClass, overrun), NULL, NULL,
       g_cclosure_marshal_VOID__VOID, G_TYPE_NONE, 0);
+  /**
+   * GstQueue::pushing:
+   * @queue: the queue instance
+   *
+   * Reports when the queue has enough data to start pushing data again on the
+   * source pad.
+   */
+  gst_queue_signals[SIGNAL_PUSHING] =
+      g_signal_new ("pushing", G_TYPE_FROM_CLASS (klass), G_SIGNAL_RUN_FIRST,
+      G_STRUCT_OFFSET (GstQueueClass, pushing), NULL, NULL,
+      g_cclosure_marshal_VOID__VOID, G_TYPE_NONE, 0);
 
   /* properties */
   g_object_class_install_property (gobject_class, ARG_CUR_LEVEL_BYTES,
@@ -299,15 +327,15 @@ gst_queue_class_init (GstQueueClass * klass)
   g_object_class_install_property (gobject_class, ARG_MAX_SIZE_BYTES,
       g_param_spec_uint ("max-size-bytes", "Max. size (kB)",
           "Max. amount of data in the queue (bytes, 0=disable)",
-          0, G_MAXUINT, 0, G_PARAM_READWRITE));
+          0, G_MAXUINT, DEFAULT_MAX_SIZE_BYTES, G_PARAM_READWRITE));
   g_object_class_install_property (gobject_class, ARG_MAX_SIZE_BUFFERS,
       g_param_spec_uint ("max-size-buffers", "Max. size (buffers)",
           "Max. number of buffers in the queue (0=disable)",
-          0, G_MAXUINT, 0, G_PARAM_READWRITE));
+          0, G_MAXUINT, DEFAULT_MAX_SIZE_BUFFERS, G_PARAM_READWRITE));
   g_object_class_install_property (gobject_class, ARG_MAX_SIZE_TIME,
       g_param_spec_uint64 ("max-size-time", "Max. size (ns)",
           "Max. amount of data in the queue (in ns, 0=disable)",
-          0, G_MAXUINT64, 0, G_PARAM_READWRITE));
+          0, G_MAXUINT64, DEFAULT_MAX_SIZE_TIME, G_PARAM_READWRITE));
 
   g_object_class_install_property (gobject_class, ARG_MIN_THRESHOLD_BYTES,
       g_param_spec_uint ("min-threshold-bytes", "Min. threshold (kB)",
@@ -334,7 +362,7 @@ gst_queue_class_init (GstQueueClass * klass)
 }
 
 static void
-gst_queue_init (GstQueue * queue)
+gst_queue_init (GstQueue * queue, GstQueueClass * g_class)
 {
   queue->sinkpad = gst_pad_new_from_static_template (&sinktemplate, "sink");
 
@@ -368,18 +396,14 @@ gst_queue_init (GstQueue * queue)
       GST_DEBUG_FUNCPTR (gst_queue_handle_src_query));
   gst_element_add_pad (GST_ELEMENT (queue), queue->srcpad);
 
-  queue->cur_level.buffers = 0; /* no content */
-  queue->cur_level.bytes = 0;   /* no content */
-  queue->cur_level.time = 0;    /* no content */
-  queue->max_size.buffers = 200;        /* 200 buffers */
-  queue->max_size.bytes = 10 * 1024 * 1024;     /* 10 MB */
-  queue->max_size.time = GST_SECOND;    /* 1 s. */
-  queue->min_threshold.buffers = 0;     /* no threshold */
-  queue->min_threshold.bytes = 0;       /* no threshold */
-  queue->min_threshold.time = 0;        /* no threshold */
-  queue->orig_min_threshold.buffers = 0;
-  queue->orig_min_threshold.bytes = 0;
-  queue->orig_min_threshold.time = 0;
+  GST_QUEUE_CLEAR_LEVEL (queue->cur_level);
+  queue->max_size.buffers = DEFAULT_MAX_SIZE_BUFFERS;
+  queue->max_size.bytes = DEFAULT_MAX_SIZE_BYTES;
+  queue->max_size.time = DEFAULT_MAX_SIZE_TIME;
+  GST_QUEUE_CLEAR_LEVEL (queue->min_threshold);
+  GST_QUEUE_CLEAR_LEVEL (queue->orig_min_threshold);
+  gst_segment_init (&queue->sink_segment, GST_FORMAT_TIME);
+  gst_segment_init (&queue->src_segment, GST_FORMAT_TIME);
 
   queue->leaky = GST_QUEUE_NO_LEAK;
   queue->srcresult = GST_FLOW_WRONG_STATE;
@@ -407,14 +431,11 @@ gst_queue_finalize (GObject * object)
     gst_mini_object_unref (data);
   }
   g_queue_free (queue->queue);
-  GST_DEBUG_OBJECT (queue, "free mutex");
   g_mutex_free (queue->qlock);
-  GST_DEBUG_OBJECT (queue, "done free mutex");
   g_cond_free (queue->item_add);
   g_cond_free (queue->item_del);
 
-  if (G_OBJECT_CLASS (parent_class)->finalize)
-    G_OBJECT_CLASS (parent_class)->finalize (object);
+  G_OBJECT_CLASS (parent_class)->finalize (object);
 }
 
 static GstCaps *
@@ -493,6 +514,90 @@ gst_queue_acceptcaps (GstPad * pad, GstCaps * caps)
   return TRUE;
 }
 
+/* calculate the diff between running time on the sink and src of the queue.
+ * This is the total amount of time in the queue. */
+static void
+update_time_level (GstQueue * queue)
+{
+  gint64 sink_time, src_time;
+
+  sink_time =
+      gst_segment_to_running_time (&queue->sink_segment, GST_FORMAT_TIME,
+      queue->sink_segment.last_stop);
+
+  src_time = gst_segment_to_running_time (&queue->src_segment, GST_FORMAT_TIME,
+      queue->src_segment.last_stop);
+
+  GST_DEBUG_OBJECT (queue, "sink %" GST_TIME_FORMAT ", src %" GST_TIME_FORMAT,
+      GST_TIME_ARGS (sink_time), GST_TIME_ARGS (src_time));
+
+  if (sink_time >= src_time)
+    queue->cur_level.time = sink_time - src_time;
+  else
+    queue->cur_level.time = 0;
+}
+
+/* take a NEWSEGMENT event and apply the values to segment, updating the time
+ * level of queue. */
+static void
+apply_segment (GstQueue * queue, GstEvent * event, GstSegment * segment)
+{
+  gboolean update;
+  GstFormat format;
+  gdouble rate, arate;
+  gint64 start, stop, time;
+
+  gst_event_parse_new_segment_full (event, &update, &rate, &arate,
+      &format, &start, &stop, &time);
+
+  /* now configure the values, we use these to track timestamps on the
+   * sinkpad. */
+  if (format != GST_FORMAT_TIME) {
+    /* non-time format, pretent the current time segment is closed with a
+     * 0 start and unknown stop time. */
+    update = FALSE;
+    format = GST_FORMAT_TIME;
+    start = 0;
+    stop = -1;
+    time = 0;
+  }
+  gst_segment_set_newsegment_full (segment, update,
+      rate, arate, format, start, stop, time);
+
+  GST_DEBUG_OBJECT (queue,
+      "configured NEWSEGMENT %" GST_SEGMENT_FORMAT, segment);
+
+  /* segment can update the time level of the queue */
+  update_time_level (queue);
+}
+
+/* take a buffer and update segment, updating the time level of the queue. */
+static void
+apply_buffer (GstQueue * queue, GstBuffer * buffer, GstSegment * segment)
+{
+  GstClockTime duration, timestamp;
+
+  timestamp = GST_BUFFER_TIMESTAMP (buffer);
+  duration = GST_BUFFER_DURATION (buffer);
+
+  /* if no timestamp is set, assume it's continuous with the previous 
+   * time */
+  if (timestamp == GST_CLOCK_TIME_NONE)
+    timestamp = segment->last_stop;
+
+  /* add duration */
+  if (duration != GST_CLOCK_TIME_NONE)
+    timestamp += duration;
+
+  GST_DEBUG_OBJECT (queue, "last_stop updated to %" GST_TIME_FORMAT,
+      GST_TIME_ARGS (timestamp));
+
+  gst_segment_set_last_stop (segment, GST_FORMAT_TIME, timestamp);
+
+  /* calc diff with other end */
+  update_time_level (queue);
+}
+
 static void
 gst_queue_locked_flush (GstQueue * queue)
 {
@@ -503,27 +608,121 @@ gst_queue_locked_flush (GstQueue * queue)
        data when flushing */
     gst_mini_object_unref (data);
   }
-  queue->cur_level.buffers = 0;
-  queue->cur_level.bytes = 0;
-  queue->cur_level.time = 0;
+  GST_QUEUE_CLEAR_LEVEL (queue->cur_level);
   queue->min_threshold.buffers = queue->orig_min_threshold.buffers;
   queue->min_threshold.bytes = queue->orig_min_threshold.bytes;
   queue->min_threshold.time = queue->orig_min_threshold.time;
+  gst_segment_init (&queue->sink_segment, GST_FORMAT_TIME);
+  gst_segment_init (&queue->src_segment, GST_FORMAT_TIME);
 
-  /* we deleted something... */
-  g_cond_signal (queue->item_del);
+  /* we deleted a lot of something */
+  GST_QUEUE_SIGNAL_DEL (queue);
+}
+
+/* enqueue an item an update the level stats */
+static void
+gst_queue_locked_enqueue (GstQueue * queue, gpointer item)
+{
+  if (GST_IS_BUFFER (item)) {
+    GstBuffer *buffer = GST_BUFFER_CAST (item);
+
+    /* add buffer to the statistics */
+    queue->cur_level.buffers++;
+    queue->cur_level.bytes += GST_BUFFER_SIZE (buffer);
+    apply_buffer (queue, buffer, &queue->sink_segment);
+
+  } else if (GST_IS_EVENT (item)) {
+    GstEvent *event = GST_EVENT_CAST (item);
+
+    switch (GST_EVENT_TYPE (event)) {
+      case GST_EVENT_EOS:
+        /* Zero the thresholds, this makes sure the queue is completely
+         * filled and we can read all data from the queue. */
+        GST_QUEUE_CLEAR_LEVEL (queue->min_threshold);
+        break;
+      case GST_EVENT_NEWSEGMENT:
+        apply_segment (queue, event, &queue->sink_segment);
+        break;
+      default:
+        break;
+    }
+  } else {
+    g_warning ("Unexpected item %p added in queue %s (refcounting problem?)",
+        item, GST_OBJECT_NAME (queue));
+    /* we can't really unref since we don't know what it is */
+    item = NULL;
+  }
+
+  if (item)
+    g_queue_push_tail (queue->queue, item);
+  GST_QUEUE_SIGNAL_ADD (queue);
+}
+
+/* dequeue an item from the queue and update level stats */
+static GstMiniObject *
+gst_queue_locked_dequeue (GstQueue * queue)
+{
+  GstMiniObject *item;
+
+  item = g_queue_pop_head (queue->queue);
+  if (item == NULL)
+    goto no_item;
+
+  if (GST_IS_BUFFER (item)) {
+    GstBuffer *buffer = GST_BUFFER_CAST (item);
+
+    GST_CAT_LOG_OBJECT (queue_dataflow, queue,
+        "retrieved buffer %p from queue", buffer);
+
+    queue->cur_level.buffers--;
+    queue->cur_level.bytes -= GST_BUFFER_SIZE (buffer);
+    apply_buffer (queue, buffer, &queue->src_segment);
+
+  } else if (GST_IS_EVENT (item)) {
+    GstEvent *event = GST_EVENT_CAST (item);
+
+    GST_CAT_LOG_OBJECT (queue_dataflow, queue,
+        "retrieved event %p from queue", event);
+
+    switch (GST_EVENT_TYPE (event)) {
+      case GST_EVENT_EOS:
+        /* queue is empty now that we dequeued the EOS */
+        GST_QUEUE_CLEAR_LEVEL (queue->cur_level);
+        break;
+      case GST_EVENT_NEWSEGMENT:
+        apply_segment (queue, event, &queue->src_segment);
+        break;
+      default:
+        break;
+    }
+  } else {
+    g_warning
+        ("Unexpected item %p dequeued from queue %s (refcounting problem?)",
+        item, GST_OBJECT_NAME (queue));
+    item = NULL;
+  }
+  GST_QUEUE_SIGNAL_DEL (queue);
+
+  return item;
+
+  /* ERRORS */
+no_item:
+  {
+    GST_CAT_LOG_OBJECT (queue_dataflow, queue, "the queue is empty");
+    return NULL;
+  }
 }
 
 static gboolean
 gst_queue_handle_sink_event (GstPad * pad, GstEvent * event)
 {
   GstQueue *queue;
-  gboolean have_eos = FALSE;
 
   queue = GST_QUEUE (GST_OBJECT_PARENT (pad));
 
   switch (GST_EVENT_TYPE (event)) {
     case GST_EVENT_FLUSH_START:
+    {
       STATUS (queue, pad, "received flush start event");
       /* forward event */
       gst_pad_push_event (queue->srcpad, event);
@@ -541,7 +740,9 @@ gst_queue_handle_sink_event (GstPad * pad, GstEvent * event)
       gst_pad_pause_task (queue->srcpad);
       GST_CAT_LOG_OBJECT (queue_dataflow, queue, "loop stopped");
       goto done;
+    }
     case GST_EVENT_FLUSH_STOP:
+    {
       STATUS (queue, pad, "received flush stop event");
       /* forward event */
       gst_pad_push_event (queue->srcpad, event);
@@ -559,37 +760,29 @@ gst_queue_handle_sink_event (GstPad * pad, GstEvent * event)
 
       STATUS (queue, pad, "after flush");
       goto done;
-    case GST_EVENT_EOS:
-      STATUS (queue, pad, "received EOS");
-      have_eos = TRUE;
-      break;
+    }
     default:
       if (GST_EVENT_IS_SERIALIZED (event)) {
-        /* we put the event in the queue, we don't have to act ourselves */
-        GST_CAT_LOG_OBJECT (queue_dataflow, queue,
-            "adding event %p of type %d", event, GST_EVENT_TYPE (event));
+        /* serialized events go in the queue */
+        GST_QUEUE_MUTEX_LOCK_CHECK (queue, out_flushing);
+        gst_queue_locked_enqueue (queue, event);
+        GST_QUEUE_MUTEX_UNLOCK (queue);
       } else {
+        /* non-serialized events are passed upstream. */
         gst_pad_push_event (queue->srcpad, event);
-        goto done;
       }
       break;
   }
-
-  GST_QUEUE_MUTEX_LOCK (queue);
-  if (have_eos) {
-    /* Zero the thresholds, this makes sure the queue is completely
-     * filled and we can read all data from the queue. */
-    queue->min_threshold.buffers = 0;
-    queue->min_threshold.bytes = 0;
-    queue->min_threshold.time = 0;
-  }
-  g_queue_push_tail (queue->queue, event);
-  g_cond_signal (queue->item_add);
-  GST_QUEUE_MUTEX_UNLOCK (queue);
-
 done:
-
   return TRUE;
+
+  /* ERRORS */
+out_flushing:
+  {
+    GST_QUEUE_MUTEX_UNLOCK (queue);
+    gst_buffer_unref (event);
+    return FALSE;
+  }
 }
 
 static gboolean
@@ -615,19 +808,24 @@ gst_queue_is_filled (GstQueue * queue)
               queue->cur_level.time >= queue->max_size.time)));
 }
 
-
 static GstFlowReturn
 gst_queue_chain (GstPad * pad, GstBuffer * buffer)
 {
   GstQueue *queue;
+  GstClockTime duration, timestamp;
 
   queue = GST_QUEUE (GST_OBJECT_PARENT (pad));
 
   /* we have to lock the queue since we span threads */
   GST_QUEUE_MUTEX_LOCK_CHECK (queue, out_flushing);
 
+  timestamp = GST_BUFFER_TIMESTAMP (buffer);
+  duration = GST_BUFFER_DURATION (buffer);
+
   GST_CAT_LOG_OBJECT (queue_dataflow, queue,
-      "adding buffer %p of size %d", buffer, GST_BUFFER_SIZE (buffer));
+      "received buffer %p of size %d, time %" GST_TIME_FORMAT ", duration %"
+      GST_TIME_FORMAT, buffer, GST_BUFFER_SIZE (buffer),
+      GST_TIME_ARGS (timestamp), GST_TIME_ARGS (duration));
 
   /* We make space available if we're "full" according to whatever
    * the user defined as "full". Note that this only applies to buffers.
@@ -637,104 +835,60 @@ gst_queue_chain (GstPad * pad, GstBuffer * buffer)
     g_signal_emit (G_OBJECT (queue), gst_queue_signals[SIGNAL_OVERRUN], 0);
     GST_QUEUE_MUTEX_LOCK_CHECK (queue, out_flushing);
 
+    /* we recheck, the signal could have changed the thresholds */
+    if (!gst_queue_is_filled (queue))
+      break;
+
     /* how are we going to make space for this buffer? */
     switch (queue->leaky) {
-        /* leak current buffer */
       case GST_QUEUE_LEAK_UPSTREAM:
+        /* leak current buffer */
         GST_CAT_DEBUG_OBJECT (queue_dataflow, queue,
             "queue is full, leaking buffer on upstream end");
         /* now we can clean up and exit right away */
         goto out_unref;
+      case GST_QUEUE_LEAK_DOWNSTREAM:
+      {
+        /* for as long as the queue is filled, dequeue an item and discard 
+         * it. */
+        do {
+          GstMiniObject *leak;
 
-        /* leak first buffer in the queue */
-      case GST_QUEUE_LEAK_DOWNSTREAM:{
-        /* this is a bit hacky. We'll manually iterate the list
-         * and find the first buffer from the head on. We'll
-         * unref that and "fix up" the GQueue object... */
-        GList *item;
-        GstMiniObject *leak = NULL;
+          leak = gst_queue_locked_dequeue (queue);
+          /* there is nothing to dequeue and the queue is still filled.. This
+           * should not happen. */
+          g_assert (leak != NULL);
 
-        if (!gst_queue_is_filled (queue)) {
-          /* Queue was emptied while we sent out the signal, so no need to drop */
           GST_CAT_DEBUG_OBJECT (queue_dataflow, queue,
-              "queue emptied while emitting signal, not leaking buffer");
-          break;
-        }
-
-        GST_CAT_DEBUG_OBJECT (queue_dataflow, queue,
-            "queue is full, leaking buffer on downstream end");
-
-        for (item = g_queue_peek_head_link (queue->queue); item;
-            item = item->next) {
-          if (GST_IS_BUFFER (item->data)) {
-            leak = item->data;
-            break;
-          }
-        }
-
-        /* if we didn't find anything, it means we have no buffers
-         * in here. That cannot happen, since we had >= 1 bufs */
-        g_assert (leak);
-
-        /* Now remove the link from the queue */
-        g_queue_delete_link (queue->queue, item);
-
-        /* and unref the buffer at the end. Twice, because we keep a ref
-         * to make things read-only. Also keep our list uptodate. */
-        queue->cur_level.bytes -= GST_BUFFER_SIZE (leak);
-        queue->cur_level.buffers--;
-        if (GST_BUFFER_DURATION (leak) != GST_CLOCK_TIME_NONE)
-          queue->cur_level.time -= GST_BUFFER_DURATION (leak);
-
-        gst_buffer_unref (leak);
+              "queue is full, leaking item %p on downstream end", leak);
+          gst_buffer_unref (leak);
+        } while (gst_queue_is_filled (queue));
         break;
       }
-
       default:
         g_warning ("Unknown leaky type, using default");
         /* fall-through */
+      case GST_QUEUE_NO_LEAK:
+      {
+        GST_CAT_DEBUG_OBJECT (queue_dataflow, queue,
+            "queue is full, waiting for free space");
 
         /* don't leak. Instead, wait for space to be available */
-      case GST_QUEUE_NO_LEAK:
-        STATUS (queue, pad, "pre-full wait");
+        do {
+          /* for as long as the queue is filled, wait till an item was deleted. */
+          GST_QUEUE_WAIT_DEL_CHECK (queue, out_flushing);
+        } while (gst_queue_is_filled (queue));
 
-        /* we recheck, the signal could have changed the thresholds */
-        while (gst_queue_is_filled (queue)) {
-          STATUS (queue, pad,
-              "waiting for item_del signal from thread using qlock");
-          g_cond_wait (queue->item_del, queue->qlock);
-
-          if (queue->srcresult != GST_FLOW_OK)
-            goto out_flushing;
-
-          /* if there's a pending state change for this queue
-           * or its manager, switch back to iterator so bottom
-           * half of state change executes */
-          STATUS (queue, pad,
-              "received item_del signal from thread using qlock");
-        }
-
-        STATUS (queue, pad, "post-full wait");
         GST_QUEUE_MUTEX_UNLOCK (queue);
         g_signal_emit (G_OBJECT (queue), gst_queue_signals[SIGNAL_RUNNING], 0);
         GST_QUEUE_MUTEX_LOCK_CHECK (queue, out_flushing);
-
         break;
+      }
     }
   }
 
-  g_queue_push_tail (queue->queue, buffer);
-
-  /* add buffer to the statistics */
-  queue->cur_level.buffers++;
-  queue->cur_level.bytes += GST_BUFFER_SIZE (buffer);
-  if (GST_BUFFER_DURATION (buffer) != GST_CLOCK_TIME_NONE)
-    queue->cur_level.time += GST_BUFFER_DURATION (buffer);
-
-  STATUS (queue, pad, "+ level");
-
-  GST_CAT_LOG_OBJECT (queue_dataflow, queue, "signalling item_add");
-  g_cond_signal (queue->item_add);
+  /* put buffer in queue now */
+  gst_queue_locked_enqueue (queue, buffer);
   GST_QUEUE_MUTEX_UNLOCK (queue);
 
   return GST_FLOW_OK;
@@ -755,148 +909,111 @@ out_flushing:
     GST_CAT_LOG_OBJECT (queue_dataflow, queue,
         "exit because task paused, reason: %s", gst_flow_get_name (ret));
     GST_QUEUE_MUTEX_UNLOCK (queue);
-
     gst_buffer_unref (buffer);
 
     return ret;
   }
 }
 
-static gboolean
+/* dequeue an item from the queue an push it downstream. This functions returns
+ * the result of the push. */
+static GstFlowReturn
 gst_queue_push_one (GstQueue * queue)
 {
-  gboolean restart = TRUE;
+  GstFlowReturn result = GST_FLOW_OK;
   GstMiniObject *data;
 
-  /* There's something in the list now, whatever it is */
-  data = g_queue_pop_head (queue->queue);
-  GST_CAT_LOG_OBJECT (queue_dataflow, queue,
-      "retrieved data %p from queue", data);
+  data = gst_queue_locked_dequeue (queue);
+  if (data == NULL)
+    goto no_item;
 
   if (GST_IS_BUFFER (data)) {
-    GstFlowReturn result;
-
-    /* Update statistics */
-    queue->cur_level.buffers--;
-    queue->cur_level.bytes -= GST_BUFFER_SIZE (data);
-    if (GST_BUFFER_DURATION (data) != GST_CLOCK_TIME_NONE)
-      queue->cur_level.time -= GST_BUFFER_DURATION (data);
+    GstBuffer *buffer = GST_BUFFER_CAST (data);
 
     GST_QUEUE_MUTEX_UNLOCK (queue);
-    result = gst_pad_push (queue->srcpad, GST_BUFFER (data));
+
+    result = gst_pad_push (queue->srcpad, buffer);
+
     /* need to check for srcresult here as well */
     GST_QUEUE_MUTEX_LOCK_CHECK (queue, out_flushing);
-    /* else result of push indicates what happens */
-    if (result != GST_FLOW_OK) {
-      const gchar *flowname;
-
-      flowname = gst_flow_get_name (result);
-
-      queue->srcresult = result;
-
-      GST_DEBUG_OBJECT (queue, "pausing queue, reason %s", flowname);
-      gst_pad_pause_task (queue->srcpad);
-    }
   } else if (GST_IS_EVENT (data)) {
-    if (GST_EVENT_TYPE (data) == GST_EVENT_EOS) {
-      queue->cur_level.buffers = 0;
-      queue->cur_level.bytes = 0;
-      queue->cur_level.time = 0;
-      /* all incomming data is now unexpected */
-      queue->srcresult = GST_FLOW_UNEXPECTED;
-      /* and we don't need to process anymore */
-      GST_DEBUG_OBJECT (queue, "pausing queue, we're EOS now");
-      gst_pad_pause_task (queue->srcpad);
-      restart = FALSE;
-    }
+    GstEvent *event = GST_EVENT_CAST (data);
+    GstEventType type = GST_EVENT_TYPE (event);
+
     GST_QUEUE_MUTEX_UNLOCK (queue);
-    gst_pad_push_event (queue->srcpad, GST_EVENT (data));
+
+    gst_pad_push_event (queue->srcpad, event);
+
     GST_QUEUE_MUTEX_LOCK_CHECK (queue, out_flushing);
-    if (restart == TRUE)
-      return TRUE;
-  } else {
-    g_warning ("Unexpected object in queue %s (refcounting problem?)",
-        GST_OBJECT_NAME (queue));
+    /* if we're EOS, return UNEXPECTED so that the task pauses. */
+    if (type == GST_EVENT_EOS)
+      result = GST_FLOW_UNEXPECTED;
   }
+  return result;
 
-  STATUS (queue, queue->srcpad, "after _get()");
-
-  GST_CAT_LOG_OBJECT (queue_dataflow, queue, "signalling item_del");
-  g_cond_signal (queue->item_del);
-
-  return FALSE;
-
+  /* ERRORS */
+no_item:
+  {
+    GST_CAT_LOG_OBJECT (queue_dataflow, queue,
+        "exit because we have no item in the queue");
+    return GST_FLOW_ERROR;
+  }
 out_flushing:
-  gst_pad_pause_task (queue->srcpad);
-  GST_CAT_LOG_OBJECT (queue_dataflow, queue,
-      "exit because task paused, reason:  %s",
-      gst_flow_get_name (queue->srcresult));
-
-  return FALSE;                 /* FALSE == no restart */
+  {
+    GST_CAT_LOG_OBJECT (queue_dataflow, queue, "exit because we are flushing");
+    return GST_FLOW_WRONG_STATE;
+  }
 }
 
 static void
 gst_queue_loop (GstPad * pad)
 {
   GstQueue *queue;
+  GstFlowReturn ret;
 
   queue = GST_QUEUE (GST_PAD_PARENT (pad));
 
   /* have to lock for thread-safety */
   GST_QUEUE_MUTEX_LOCK_CHECK (queue, out_flushing);
 
-restart:
   while (gst_queue_is_empty (queue)) {
     GST_QUEUE_MUTEX_UNLOCK (queue);
     g_signal_emit (G_OBJECT (queue), gst_queue_signals[SIGNAL_UNDERRUN], 0);
     GST_QUEUE_MUTEX_LOCK_CHECK (queue, out_flushing);
 
-    STATUS (queue, pad, "pre-empty wait");
     /* we recheck, the signal could have changed the thresholds */
     while (gst_queue_is_empty (queue)) {
-      STATUS (queue, pad, "waiting for item_add");
-
-      GST_LOG_OBJECT (queue, "doing g_cond_wait using qlock from thread %p",
-          g_thread_self ());
-      g_cond_wait (queue->item_add, queue->qlock);
-
-      /* we released the lock in the g_cond above so we might be
-       * flushing now */
-      if (queue->srcresult != GST_FLOW_OK)
-        goto out_flushing;
-
-      GST_LOG_OBJECT (queue, "done g_cond_wait using qlock from thread %p",
-          g_thread_self ());
-      STATUS (queue, pad, "got item_add signal");
+      GST_QUEUE_WAIT_ADD_CHECK (queue, out_flushing);
     }
-
-    STATUS (queue, pad, "post-empty wait");
     GST_QUEUE_MUTEX_UNLOCK (queue);
+
     g_signal_emit (G_OBJECT (queue), gst_queue_signals[SIGNAL_RUNNING], 0);
+    g_signal_emit (G_OBJECT (queue), gst_queue_signals[SIGNAL_PUSHING], 0);
+
     GST_QUEUE_MUTEX_LOCK_CHECK (queue, out_flushing);
   }
 
-  if (gst_queue_push_one (queue))
-    goto restart;
+  ret = gst_queue_push_one (queue);
+  queue->srcresult = ret;
+  if (ret != GST_FLOW_OK)
+    goto out_flushing;
 
   GST_QUEUE_MUTEX_UNLOCK (queue);
 
   return;
 
+  /* ERRORS */
 out_flushing:
   {
     gst_pad_pause_task (queue->srcpad);
     GST_CAT_LOG_OBJECT (queue_dataflow, queue,
-        "exit because task paused, reason:  %s",
-        gst_flow_get_name (queue->srcresult));
-
+        "pause task, reason:  %s", gst_flow_get_name (queue->srcresult));
+    GST_QUEUE_SIGNAL_DEL (queue);
     GST_QUEUE_MUTEX_UNLOCK (queue);
-
     return;
   }
 }
 
-
 static gboolean
 gst_queue_handle_src_event (GstPad * pad, GstEvent * event)
 {
