@@ -638,6 +638,38 @@ GST_START_TEST (controller_interpolate_linear)
 
 GST_END_TEST;
 
+/* make sure we don't crash when someone sets an unsupported interpolation
+ * mode */
+GST_START_TEST (controller_interpolate_unimplemented)
+{
+  GstController *ctrl;
+  GstElement *elem;
+
+  gst_controller_init (NULL, NULL);
+
+  elem = gst_element_factory_make ("testmonosource", "test_source");
+
+  /* that property should exist and should be controllable */
+  ctrl = gst_controller_new (G_OBJECT (elem), "ulong", NULL);
+  fail_unless (ctrl != NULL, NULL);
+
+  /* set unsupported interpolation mode */
+  gst_controller_set_interpolation_mode (ctrl, "ulong", GST_INTERPOLATE_CUBIC);
+
+  /* set another unsupported interpolation mode */
+  gst_controller_set_interpolation_mode (ctrl, "ulong",
+      GST_INTERPOLATE_QUADRATIC);
+
+  /* set completely bogus interpolation mode */
+  gst_controller_set_interpolation_mode (ctrl, "ulong",
+      (GstInterpolateMode) 93871);
+
+  g_object_unref (ctrl);
+  gst_object_unref (elem);
+}
+
+GST_END_TEST;
+
 /* test _unset() */
 GST_START_TEST (controller_unset)
 {
@@ -851,6 +883,8 @@ GST_START_TEST (controller_misc)
   fail_unless (gst_controller_set_from_list (ctrl, "ulong", list));
 
   /* allocated GstTimedValue now belongs to the controller, but list not */
+  g_value_unset (&tval->value);
+  g_free (tval);
   g_slist_free (list);
   g_object_unref (ctrl);
   gst_object_unref (elem);
@@ -936,6 +970,55 @@ GST_START_TEST (controller_refcount_new_list)
 
 GST_END_TEST;
 
+
+/* test retrieval of an array of values with get_value_array() */
+GST_START_TEST (controller_interpolate_linear_value_array)
+{
+  GstController *ctrl;
+  GstElement *elem;
+  gboolean res;
+  GValue val_ulong = { 0, };
+  GstValueArray values = { 0, };
+
+  gst_controller_init (NULL, NULL);
+
+  elem = gst_element_factory_make ("testmonosource", "test_source");
+
+  /* that property should exist and should be controllable */
+  ctrl = gst_controller_new (G_OBJECT (elem), "ulong", NULL);
+  fail_unless (ctrl != NULL, NULL);
+
+  /* set interpolation mode */
+  gst_controller_set_interpolation_mode (ctrl, "ulong", GST_INTERPOLATE_LINEAR);
+
+  /* set control values */
+  g_value_init (&val_ulong, G_TYPE_ULONG);
+  g_value_set_ulong (&val_ulong, 0);
+  res = gst_controller_set (ctrl, "ulong", 0 * GST_SECOND, &val_ulong);
+  fail_unless (res, NULL);
+  g_value_set_ulong (&val_ulong, 100);
+  res = gst_controller_set (ctrl, "ulong", 2 * GST_SECOND, &val_ulong);
+  fail_unless (res, NULL);
+
+  /* now pull in values for some timestamps */
+  values.property_name = "ulong";
+  values.nbsamples = 3;
+  values.sample_interval = GST_SECOND;
+  values.values = (gpointer) g_new (gulong, 3);
+
+  fail_unless (gst_controller_get_value_array (ctrl, 0, &values));
+  fail_unless_equals_int (((gulong *) values.values)[0], 0);
+  fail_unless_equals_int (((gulong *) values.values)[1], 50);
+  fail_unless_equals_int (((gulong *) values.values)[2], 100);
+
+  GST_INFO ("controller->ref_count=%d", G_OBJECT (ctrl)->ref_count);
+  g_free (values.values);
+  g_object_unref (ctrl);
+  gst_object_unref (elem);
+}
+
+GST_END_TEST;
+
 static Suite *
 gst_controller_suite (void)
 {
@@ -958,11 +1041,13 @@ gst_controller_suite (void)
   tcase_add_test (tc, controller_interpolate_none);
   tcase_add_test (tc, controller_interpolate_trigger);
   tcase_add_test (tc, controller_interpolate_linear);
+  tcase_add_test (tc, controller_interpolate_unimplemented);
   tcase_add_test (tc, controller_unset);
   tcase_add_test (tc, controller_unset_all);
   tcase_add_test (tc, controller_live);
   tcase_add_test (tc, controller_helper_any_gobject);
   tcase_add_test (tc, controller_misc);
+  tcase_add_test (tc, controller_interpolate_linear_value_array);
 
   return s;
 }
